@@ -14,20 +14,32 @@ class MCMC:
     """
     Monte Carlo sampler class.
 
-    Parameters
-    ----------
-    walkers : int, optional
-        Number of walkers, by default 32
-    steps : int, optional
-        Number of steps, by default 1600
-    progress : bool, optional
-        Print progress bar, by default True
+    :Example:
+        ::
+
+            from tweezepy import AV,downsampled_trace
+            trace = downsampled_trace
+            av = AV(trace, 100)
+            av.mlefit()
+            av.mcmc()
+            av.sample_plot()
+            av.corner_plot()
+    
     """
     def __init__(self, walkers = 32, steps = 1600, progress = True,**kwargs):
-        try:
-            import emcee
-        except ImportError:
-            RuntimeError("Monte Carlo sampling requires the emcee package.")
+        """
+        Monte Carlo sampler
+
+        Parameters
+        ----------
+        walkers : int, optional
+            Number of walkers, by default 32
+        steps : int, optional
+            Number of steps, by default 1600
+        progress : bool, optional
+            Print progress bar, by default True
+        """
+        import emcee
         self.walkers = walkers
         self.steps = steps
 
@@ -43,7 +55,8 @@ class MCMC:
                     fig=None,
                     labels = [],
                     fig_kwgs={},
-                    ax_kwgs={}):
+                    ax_kwgs={},
+                    no_display = False):
         """
         Plot the accepted Monte Carlo samples.
 
@@ -60,12 +73,13 @@ class MCMC:
 
         Returns
         -------
-        fig : Figure
-            Figure object.
-        axes : Axes
-            Axes object.
+        (fig, axes) : tuple
+            Figure and axes objects.
         """
         try:
+            import matplotlib
+            if no_display:
+                matplotlib.use('Agg')
             import matplotlib.pyplot as plt
         except ImportError:
             raise RuntimeError("Matplotlib is required for plotting.")
@@ -101,11 +115,6 @@ class MCMC:
             Number of "burn-in" steps to discard, by default 100
         thin : int, optional
             N, by default 10
-        
-        Return
-        ------
-        errors : array
-            Errors from Monte Carlo sampling.
         """
         tau = max(self.autocorr_time)
 
@@ -136,10 +145,8 @@ class MCMC:
 
         Returns
         -------
-        fig : Figure
-            Figure object.
-        axes : Axes
-            Axes object.
+        fig,ax
+            Figure and axes objects.
         """
         try:
             import corner
@@ -159,81 +166,55 @@ class MCMC:
         ax = fig.get_axes()
         for a in ax:
             if a.get_xlabel():
-                a.ticklabel_format(axis='x',style='sci',scilimits=(0,0),useMathText=True)
-                t = a.get_xaxis().get_offset_text()
-                t.set_position((1.1,.25))
-                #t.set_xposition((-.5,0.9))
-                #a.ticklabel_format()
+                a.ticklabel_format(axis='x',style='sci',scilimits=(0,0))
             if a.get_ylabel():
-                a.ticklabel_format(axis='y',style='sci',scilimits=(0,0),useMathText=True)
-                t = a.get_yaxis().get_offset_text()
-                t.set_position((-.3,0.9))
+                a.ticklabel_format(axis='y',style='sci',scilimits=(0,0))
+                a.get_yaxis().get_offset_text().set_position((-.5,0.9))
         return fig,ax
         
 class MLEfit(MCMC):
     """
     Perform maximum likelihood estimation and uncertainty calculations.
-
-    Parameters
-    ----------
-    pedantic : bool, optional
-        Ignore unhelpful warnings,
-        by default True
-    scale_covar : bool, optional
-        Whether to scale standard errors by reduced chi-squared,
-        by default False
-    fit_kwargs : dict, optional
-        Disctionary of keyword arguments passed to scipy.optimize.minimize, 
-        by default {}.
-    
-    Attributes
-    ----------
-    names : list
-        Fit function parameter names.
-    params : array
-        Parameter values.
-    std_errors : array
-        Parameter uncertainties.
-    chi2 : float
-        Chi-squared value.
-    redchi2 : float
-        Reduced chi-squared value.
-    AIC : float 
-        Akaike information criterion. 
-    AICc : float 
-        Corrected Akaike information criterion. 
     """
-    def __init__(self, pedantic = True, scale_covar = False, minimizer_kwargs = {}):
+    def __init__(self, pedantic = True, scale_covar = False,fit_kwargs = {'method':'Nelder-Mead'}):
+        """
+        Parameters
+        ----------
+        pedantic : bool, optional
+            Ignore unhelpful warnings, by default True
+        scale_covar : bool, optional
+            Whether to scale standard errors by reduced chi-squared, by default False
+        fit_kwargs : dict, optional
+            Fitting keywork arguments to send to scipy.minimize, by default {'method':'Nelder-Mead'}
+        """
         if pedantic == False:
             np.seterr('warn')
         elif pedantic == True:
             np.seterr('ignore')
         # Fancy way of determining fit param names        
         names = signature(self.func).parameters # inspect fit function parameters
-        self.names = list(names.keys()) # make list of parameter names
+        self.names = list(names.keys()); # make list of parameter names
         # Data
-        shape = self.data['shape']
-        y = self.data['y']
-        yerr = self.data['yerr']
+        data = self.data
+        shape,y,yerr = data['shape'],data['y'],data['yerr']
         self.ndata = len(y)
         # Log likelihood
-        self.gd = Gamma_Distribution(shape,y)
         self.logL = lambda p: self.gd.logpdf(self.func(*p)).sum()
         # Negative log likelihood
         self.negLL =  lambda p: -self.logL(p)
         # Use automatic differentiation to calculate hessian
         hess = hessian(self.negLL)
         # Minimize negative log likelihood
-        self.fit = minimize(self.negLL,x0=self.guess,method = 'Nelder-Mead',**minimizer_kwargs)
+        self.fit = minimize(self.negLL,x0=self.guess,**fit_kwargs)
         # Save minimizer fit results
         self.params = self.fit['x']
         self.nparams = len(self.fit['x'])
         self.success = self.fit['success']
         # Collect results into dictionary
-        results = {}
+        self.results = {}
         # Throw warning if fit fails
         if not self.success:
-            print('MLE fitting failed to converge. %s'%self.fit['message'])
+            print('MLE fitting failed. %s'%self.fit['message'])
             #self.params = np.array([float('nan') for i in range(self.nparams)])
             #self.std_errors = np.array([float('nan') for i in range(self.nparams)])
         # Compute standard errors by inverting the Hessian
@@ -244,34 +225,31 @@ class MLEfit(MCMC):
         self.std_errors = np.sqrt(np.diag(self.cov))
         # Calculate fit values
         yfit = self.func(*self.params); 
-        self.fit_data = self.data.copy()
+        self.fit_data = data.copy()
         self.data['yfit'] = yfit
         # Calculate residuals, chi2, and reduced chi2
-        residuals = (y-yfit)/yerr; self.residuals = residuals
+        residuals = (y-yfit)/yerr; 
         self.data['residuals'] = residuals
-        self.chi2 = np.power(residuals,2).sum(); results['chi2'] = self.chi2
+        self.chi2 = np.power(residuals,2).sum(); self.results['chi2'] = self.chi2
         self.nfree = self.ndata-self.nparams
-        self.redchi2 = self.chi2/self.nfree; results['redchi2'] = self.redchi2
+        self.redchi2 = self.chi2/self.nfree; self.results['redchi2'] = self.redchi2
         # Scale errors by reduched chi-squared value
         if scale_covar:
             self.std_errors *= self.redchi2
         # Collect errors into results
         for i,p in enumerate(self.names):
-            results['%s'%p] = self.params[i]
-            results['%s_error'%p] = self.std_errors[i]
+            self.results['%s'%p] = self.params[i]
+            self.results['%s_error'%p] = self.std_errors[i]
         # Calculate fit support and p-value
         ks = stats.kstest(residuals,'chi2',args=(self.nfree,))
         self.support,self.p = ks
-        results['support'] = self.support
-        results['p-value'] = self.p
-        
+        self.results['support'] = self.support
+        self.results['p-value'] = self.p
         # Calculate loglikelihood
         self.loglikelihood = self.logL(self.params)
         # Calculate AIC, AICc, and BIC
-        self.AIC = 2.*(self.nparams-self.loglikelihood); results['AIC'] = self.AIC
-        self.AICc = self.AIC + (2*(pow(self.nparams,2)+self.nparams))/(self.ndata-self.nparams-1) ; results['AICc'] = self.AICc
-        # Save results into private attribute
-        self._results = results
+        self.AIC = 2.*(self.nparams-self.loglikelihood); self.results['AIC'] = self.AIC
+        self.AICc = self.AIC + (2*(pow(self.nparams,2)+self.nparams))/(self.ndata-self.nparams-1)
 
     def mcmc(self, walkers = 32, steps = 2000, discard = 100, thin = 10):
         """
@@ -295,145 +273,43 @@ class MLEfit(MCMC):
         for i,name in enumerate(self.names):
             std_l, median, std_u = percentiles[i]
             self.mcmc_params[i] = median
-            self._results['%s_mcmc'%name] = median
+            self.results['%s_mcmc'%name] = median
             std_error = 0.5 * (std_u - std_l)
             self.mcmc_std_errors[i] = std_error
-            self._results['%s_mcmc_error'%name] = 0.5 * (std_u - std_l)
-    @property
-    def results(self):
-        """
-        Dictionary of MLE fit results.
-
-        Returns
-        -------
-        dict
-            Dictionary of MLE fit results.
-        """
-        return self._results
+            self.results['%s_mcmc_error'%name] = 0.5 * (std_u - std_l)
 
 class Gamma_Distribution:
-    """Gamma probability distribution class.
-
-    Parameters
-    ----------
-    shape : np.array
-        Shape parameter.
-    yhat : np.array
-        Experimental values. 
-    
-    Attributes
-    ----------
-    shape : array-like
-        Shape parameters.
-    yhat : array-like
-        Experimnetal values
+    """
+    Gamma probability distribution for AV and PSD.
     """
     def __init__(self,shape,yhat):
+        """
+        Parameters
+        ----------
+        shape : np.array
+            Shape parameter.
+        yhat : np.array
+            Experimental values. 
+        """
         self.yhat = yhat
         self.shape = shape
-    def scale(self,ytrue):
-        """ Calculate scale parameters.
-        
-        Parameters
-        ----------
-        ytrue : array-like
-            True/theoretical values.
-
-        Returns
-        -------
-        scale : array-like
-            Scale parameters.
-        """
-        return ytrue/self.shape 
-    def std(self,ytrue):
-        """ Calculate standard deviation.
-
-        Parameters
-        ----------
-        ytrue : array-like
-            True/theoretical values.
-
-        Returns
-        -------
-        std : array-like
-            Standard deviations.
-        """
-        scale = self.scale(ytrue)
+    def scale(self,y):
+        return y/self.shape 
+    def std(self,y):
+        scale = self.scale(y)
         return stats.gamma.std(self.shape, scale = scale)
-    def pdf(self,ytrue):
-        """ Calculate probability distribution function.
-
-        Parameters
-        ----------
-        ytrue : array-like
-            True/theoretical values.
-
-        Returns
-        -------
-        pdf : callable
-            Probability distribution function.
-        """
-        scale = self.scale(ytrue)
+    def pdf(self,y):
+        scale = self.scale(y)
         return gamma.pdf(self.yhat/scale,self.shape)/scale
-    def logpdf(self,ytrue):
-        """ Log of the probability distribution function.
-
-        Parameters
-        ----------
-        ytrue : array-like
-            True/theoretical values.
-
-        Returns
-        -------
-        logpdf : callable
-            Log of the probability distribution function.
-        """
-        scale = self.scale(ytrue)
+    def logpdf(self,y):
+        scale = self.scale(y)
         return gamma.logpdf(self.yhat/scale,self.shape) - np.log(scale)
-    def cdf(self,ytrue):
-        """ Cumulative distribution function.
-
-        Parameters
-        ----------
-        ytrue : array-like
-            True/theoretical values.
-
-        Returns
-        -------
-        cdf : callable
-            Cumulative distribution function.
-        """
-        scale = self.scale(ytrue)
+    def cdf(self,y):
+        scale = self.scale(y)
         return stats.gamma.cdf(self.yhat,self.shape,scale=scale)
-    def logcdf(self, ytrue):
-        """ Log of cumulative distribution function.
-
-        Parameters
-        ----------
-        ytrue : array-like
-            True/theoretical values.
-
-        Returns
-        -------
-        logcdf : callable
-            Log of cumulative distribution function.
-        """
-        scale = self.scale(ytrue)
+    def logcdf(self, y):
+        scale = self.scale(y)
         return gamma.logcdf(self.yhat,self.shape,scale = scale)
-    def interval(self,ytrue,alpha = 0.95):
-        """
-
-        Parameters
-        ----------
-        ytrue : array-like
-            True/theoretical values.
-        alpha : float, optional
-            Probability that a random variable will be drawn from the returned range. Each value should be in the range [0, 1]., by default 0.95
-
-        Returns
-        -------
-        Interval : callable
-            Endpoints of the range that contains fraction alpha [0, 1] of the distribution.
-        """
-        scale = self.scale(ytrue)
+    def interval(self,y,alpha = 0.95):
+        scale = self.scale(y)
         return stats.gamma.interval(alpha,self.shape,scale = scale)
\ No newline at end of file
