@@ -1,44 +1,23 @@
-"""
-This module performs operations associated with calculating the Allan variance. 
-
-Adapted from allantools https://github.com/aewallin/allantools.
-"""
-
 import numpy as np
 import scipy
-
-from warnings import warn
+import warnings 
 
 def m_generator(N,taus = 'octave'):
-    """ 
-    Generates averaging factor for calculating Allan variance.
-
-    Parameters
-    ----------
-    N : int
-        Number of points in trajectory
-    taus : str, optional
-        Type of sampling, by default 'octave'
-
-    Returns
-    -------
-    m : np.array
-        Averaging factor.
-    """
     assert taus in ['all','octave','decade'], "taus must be either all, octave, or decade."
     if taus == 'all':
         # all-tau sampling not particularly useful but why not?
-        maxn = N//2
-        m = np.linspace(1.0,maxn,maxn,dtype='int')
+        m = np.linspace(1.0,N,N,dtype='int')
+        m = m[m<=N//2]
     elif taus == 'octave':
         # octave sampling break bin sizes, m, into powers of 2^n
         maxn = int(np.floor(np.log2(N/2))) # m =< N/2
         m = np.logspace(0,maxn-1,maxn,base=2,dtype='int')  #bin sizes
     elif taus == 'decade':
         # again not particularly useful, but why not?
-        maxn = int(np.floor(np.log10(N/2)))
+        maxn = int(np.floor(np.log10(N)))
         m = [np.array([1,2,4])*k for k in np.logspace(0,maxn,maxn+1,base=10,dtype='int')]
         m = np.ravel(m)
+        m = m[m<=N//2]
     return m
 
 def calc_avar(phase,rate,mj,step):
@@ -50,43 +29,31 @@ def calc_avar(phase,rate,mj,step):
     avar = np.sum(v_arr*v_arr)/(2.*nmin*pow(mj/rate,2))
     return avar
 
-def avar(data, rate = 1.0, taus = 'octave', overlapping = True, edf = 'approx'):
+def avar(data,rate = 1.0,taus = 'octave', overlapping = True, edf = 'approx'):
     """
-    Calculates the standard and overlapping Allan variance.
-    Takes an array of bead positions. Returns the taus, edfs, and oavs.
-
-    .. math::
-        \\sigma^2_{AV}(\\tau) = { 1 \\over 2 (m \\tau_s )^2 (N-2m+1) }
-        \\sum_{n=0}^{N-2m} ( {z}_{n+2m} - 2z_{n+1m} + z_{n} )^2
-
-    where :math:`z_1=0`, `n=1,...,N`, and
-    
-    .. math::
-        z_n = {1 \\over m_k}
-        \\sum_{j=1}^{j-1} x_j 
-
-    where :math:`\\sigma^2_x(m\\tau_s)` is the Allan variance at an averaging time of :math:`\\tau=m_k\\tau_s`, and :math:`x_j` is the time-series of bead positions, spaced by the measurement interval :math:`\\tau_s`, with length :math:`N`.
+    Calculate standard allan variance 
+    Takes an array of bead positions.
+    Returns the taus, shapes, and oavs.
 
     Parameters
     ----------
-    data : array-like
-        1-D array of bead positions in nm.
+    data : array, series, or list
+        1-D array of numbers.
     rate : float
-        Frequency of acquisition in Hz.
+        Frequency of acquisition.
 
     Returns
     -------
-    (taus, edfs, oavs) : tuple
-        Tuple of computed values.
     taus : array
-        Observation times.
-    edfs : array
-        Equivalent degrees of freedom.
+        taus.
+    shape : array
+        shapes.
     oavs : array
-        Computed Allan variance for each tau value.
+        Allan variance.
+
     """
     assert type(overlapping) == bool, 'overlapping keyword argument should be a boolean.'
-    assert edf in ['approx','real'], 'edf keyword argument should be approx or real.'
+    assert (edf == 'approx') or (edf == 'real'), 'edf keyword argument should be approx or real.'
     rate = float(rate)
     data = np.asarray(data) # convert to numpy array
     N = len(data)
@@ -95,6 +62,11 @@ def avar(data, rate = 1.0, taus = 'octave', overlapping = True, edf = 'approx'):
     m = m[n>=2]
     taus = m/rate # tau = m*tau_c
     if edf == 'real':
+        #alpha_int =  np.array([noise_id(data,mj)[0] for mj in m])
+        #prev = np.arange(len(alpha_int))
+        #prev[np.isnan(alpha_int)] = 0
+        #prev = np.maximum.accumulate(prev)
+        #alpha_int = alpha_int[prev]
         edfs = np.empty(len(m))
         for i,mj in enumerate(m):
             if N//mj > 32:
@@ -102,12 +74,23 @@ def avar(data, rate = 1.0, taus = 'octave', overlapping = True, edf = 'approx'):
             if (alpha_int<3) and (alpha_int>-3):
                 edfs[i] = edf_greenhall(alpha_int,2,mj,N)
             else:
-                warn('Real edf failed to identify noise for %s. Falling back to approximate edf.'%mj)
+                warnings.warn('Real edf failed to identify noise for %s. Falling back to approximate edf.'%mj)
                 edfs[i] = edf_approx(N,mj)
+        #edfs = np.array([edf_greenhall(alpha_int[i],2,mj,N) if (alpha_int[i]<3) and (alpha_int[i]>-3) else edf_approx(N,mj) for i,mj in enumerate(m)])
+        #edfs = np.zeros(len(m))
+        #for i,mj in enumerate(m):
+            # autocorrelation is only good for N/mj >32
+            #if N//mj > 32:
+            #    alpha_int = noise_id(data,mj)[0]
+            #if (alpha_int < 3) and (alpha_int >-3):
+            #    edfs[i] = edf_greenhall(alpha_int,2,mj,N,overlapping=overlapping)
+            #else:
+            #    warnings.warn('Real edf failed for %s'% mj)
+            #    edfs[i] = edf_approx(N,mj)
     elif edf == 'approx':
         edfs = edf_approx(N,m)
     else:
-        warn('edf keyword argument %s not recognized.'%edf)
+        print('edf keyword argument %s not recognized.'%edf)
         raise UserWarning
     # Calculate phasedata from Eq. 18b (in erratum)
     phase = np.cumsum(data)/rate  # integrate positions, converting frequency to phase data
@@ -123,39 +106,20 @@ def avar(data, rate = 1.0, taus = 'octave', overlapping = True, edf = 'approx'):
 def totvar(data, rate=1.0, taus='octave',edf = 'approx'):
     """ Total variance.
         Better confidence at long averages for Allan variance.
-
     .. math::
-        \\sigma^2_{AV}( m\\tau_s ) = { 1 \\over 2 (m\\tau_s)^2 (N-2) }
-            \\sum_{n=2}^{N-1} ( {x}^*_{n-m} - 2x^*_{j} + x^*_{n+m} )^2
-
-    where :math:`x^*_n` is a new time-series of length :math:`3N-4` derived from the original time-series :math:`x_n` of length :math:`N` by reflection at both ends.
-
+        \\sigma^2_{TOTDEV}( m\\tau_0 ) = { 1 \\over 2 (m\\tau_0)^2 (N-2) }
+            \\sum_{i=2}^{N-1} ( {x}^*_{i-m} - 2x^*_{i} + x^*_{i+m} )^2
+    where :math:`x^*_i` is a new time-series of length :math:`3N-4`
+    derived from the original phase time-series :math:`x_n` of
+    length :math:`N` by reflection at both ends.
     Parameters
     ----------
     data: np.array
-        Bead positions in nm.
+        Bead positions.
     rate: float
-        The sampling rate for the bead positions in Hz.
-    taus: str or np.array
-        Taus spacing, options are decade, octave, or all. Alternatively, an array of tau values can be given directly.
-    
-    Returns
-    -------
-    (taus,edfs,tvars): tuple
-        Arrays of compute values.
-    taus : np.array
-        Observation times.
-    edfs : np.array
-        Equivalent degrees of freedom.
-    tvars : np.array
-        Total variances.
-
-    References
-    ----------
-    Total variance as an exact analysis of the sample variance
-    Percival, D.B and Howe, D.A.
-    Proccedings of the 1995 IEEE International Frequency Control Symposium
-    https://doi.org/10.1109/FREQ.1995.483917
+        The sampling rate for phase or frequency, in Hz
+    taus: np.array
+        Array of tau values for which to compute measurement
     """
     rate = float(rate)
     data = np.asarray(data) # make sure data is an array, not a series or list
@@ -199,7 +163,7 @@ def totvar(data, rate=1.0, taus='octave',edf = 'approx'):
             if (alpha_int <= 0) and (alpha_int >= -2):
                 edfs[idx] = edf_totdev(N,mj,alpha_int)
             else:
-                warn('Real edf failed for %s.'%mj)
+                warnings.warn('Real edf failed for %s.'%mj)
                 edfs[idx] = edf_approx(N,mj)
     return taus,edfs,tvars
 def totvar_bias(alpha_int):
@@ -220,7 +184,9 @@ def calc_totvar(x,rate,N,mid,mj):
     return var
 
 ########################################################################
-# Noise Identification Lag1 autorrelation
+# Noise Identification Lac1 autorrelation
+
+
 
 def noise_id(x,af, dmin = 0, dmax = 2):
     """
@@ -246,21 +212,15 @@ def noise_id(x,af, dmin = 0, dmax = 2):
     alpha : float
         float power-law noise
 
-    References
-    ----------
-    Power law noise identification using the lag 1 autocorrelation
-    Riley,W.J. et al.
-    18th European Frequency and Time Forum (EFTF 2004)
-    https://ieeexplore.ieee.org/document/5075021
-
     """
     # Split time series into average positions of nonoverlapping bins
     N = len(x)
     x = x[:N//af * af].reshape((N//af,af)) # cut to correct lengths
     x = np.average(x,axis=1)
     # require minimum length for time-series
-    if N < 32:
-        warn(("noise_id() Can't determine noise-ID for time-series of length= %d") %len(x))
+    if len(x) < 32:
+        #print(("noise_id() Can't determine noise-ID for"
+        #       " time-series length= %d") % len(x))
         return np.nan,np.nan
         #raise NotImplementedError
     d = 0 # number of differentiations
@@ -278,8 +238,7 @@ def noise_id(x,af, dmin = 0, dmax = 2):
             
 def edf_greenhall(alpha, d, m, N,
                   overlapping=False, modified=False, verbose=False):
-    """ Returns equivalent degrees of freedom
-        
+    """ returns Equivalent degrees of freedom - couresy of allantools
         Parameters
         ----------
         alpha: int
@@ -298,17 +257,25 @@ def edf_greenhall(alpha, d, m, N,
             True for oadev, ohdev
         modified: bool
             True for mdev, tdev
-
         Returns
         -------
         edf: float
             Equivalent degrees of freedom
-        
-        References
-        ----------
         Greenhall, Riley, 2004
         https://ntrs.nasa.gov/archive/nasa/casi.ntrs.nasa.gov/20050061319.pdf
         UNCERTAINTY OF STABILITY VARIANCES BASED ON FINITE DIFFERENCES
+        Notes
+        -----
+        Based on allantools https://github.com/aewallin/allantools
+
+        Used for the following deviations
+        (see http://www.wriley.com/CI2.pdf page 8)
+        adev()
+        oadev()
+        mdev()
+        tdev()
+        hdev()
+        ohdev()
     """
 
     if modified:
@@ -460,7 +427,8 @@ def greenhall_sz(t, F, alpha, d):
     assert(0)  # ERROR
 
 def greenhall_sx(t, F, alpha):
-    """ Eqn (8) from Greenhall2004 """
+    """ Eqn (8) from Greenhall2004
+    """
     if F == float('inf'):
         return greenhall_sw(t, alpha+2)
     a = 2*greenhall_sw(t, alpha)
@@ -471,7 +439,8 @@ def greenhall_sx(t, F, alpha):
 
 
 def greenhall_sw(t, alpha):
-    """ Eqn (7) from Greenhall2004 """
+    """ Eqn (7) from Greenhall2004
+    """
     alpha = int(alpha)
     if alpha == 2:
         return -np.abs(t)
@@ -554,9 +523,8 @@ def greenhall_table1(alpha, d):
     return table1[row_idx][col_idx]
 
 def edf_totdev(N, m, alpha):
-    """ 
-    Equivalent degrees of freedom for Total Deviation
-    NIST SP1065 page 41, Table 7
+    """ Equivalent degrees of freedom for Total Deviation
+        NIST SP1065 page 41, Table 7
     """
     alpha = int(alpha)
     if alpha in [0, -1, -2]:
@@ -572,7 +540,6 @@ def edf_totdev(N, m, alpha):
 def edf_simple(N, m, alpha, pedantic = False):
     """Equivalent degrees of freedom.
     Simple approximate formulae.
-
     Parameters
     ----------
     N : int
@@ -588,18 +555,17 @@ def edf_simple(N, m, alpha, pedantic = False):
         'rf' returns random walk frequency noise.   alpha=-2
         If the input is not recognized, it defaults to idealized, uncorrelated
         noise with (N-1) degrees of freedom.
-
+    Notes
+    -----
+       S. Stein, Frequency and Time - Their Measurement and
+       Characterization. Precision Frequency Control Vol 2, 1985, pp 191-416.
+       http://tf.boulder.nist.gov/general/pdf/666.pdf
+       
+       Modified from allantools.
     Returns
     -------
     edf : float
         Equivalent degrees of freedom
-
-    References
-    ----------
-       S. Stein, Frequency and Time - Their Measurement and
-       Characterization. Precision Frequency Control Vol 2, 1985, pp 191-416.
-       http://tf.boulder.nist.gov/general/pdf/666.pdf
-    
     """
 
     N = float(N)
@@ -634,33 +600,11 @@ def edf_simple(N, m, alpha, pedantic = False):
     else:
         edf = (N/m - 1) # assume correlated noise
         if pedantic == True:
-            warn("Noise type not recognized."
-                 " Defaulting to N/m - 1 degrees of freedom.")
+            print("Noise type not recognized."
+                  " Defaulting to N/m - 1 degrees of freedom.")
 
     return edf
 
+
 def edf_approx(N,mj):
-    """
-    Approximate edf based on the number of nonoverlapping bins.
-
-    Parameters
-    ----------
-    N : int
-        Number of bead trajectory points.
-    mj : int
-        Averaging factor
-
-    Returns
-    -------
-    edf : int
-        Equivalent degrees of freedom
-
-    References
-    ----------
-    Power spectrum and Allan variance methods for calibrating single-molecule video-tracking instruments.
-    Lansdorp, B.M. and Saleh, O.A.
-    Review of Scientific Instruments (2012)
-    https://doi.org/10.1063/1.3687431
-    """
-    edf = N//mj - 1
-    return edf
+    return N//mj-1
