@@ -1,42 +1,15 @@
-"""
-This module include functions for running bead trajectory simulations.
-
-"""
 import numpy as np
 from numba import jit
-
 @jit(nopython=True) # speeds up certain types of code
-def simulate_trace(gamma = 1.e-5,
+def simulate_trace(gamma = 1.se-5,
                    kappa = None,
                    fsim = 400,
                    sim_points = 10240,
                    seed = None):
-    """
-    Takes parameters and gives a simulated bead trajectory.
-
+    """Takes parameters and gives a simulated bead trajectory.
     .. math::
-        \Delta x = \\frac{\Delta t}{\\gamma}\left(\sqrt{\\frac{2\\gamma k_BT}{dt}} F_L-\\kappa x_{i-1}\\right)
-    .. math:: 
+        dx = \frac{dt}{\alpha}\left(\sqrt{\frac{2\alpha k_BT}{dt}} F_L-\kappa x_{i-1}\right) \\
         x_i = x_{i-1}+dx
-
-    Parameters
-    ----------
-    gamma : float, optional
-        Stokes dissipation in pNs/nm^2, by default 1.e-5
-    kappa : float, optional
-        Spring constant in pN/nm, by default None
-    fsim : int, optional
-        Simulation frequency in Hz, by default 100
-    sim_points : int, optional
-        Number of bead positions, by default 10240
-    seed : int, optional
-        Random seed, by default None
-    
-    Returns
-    -------
-    xtrace : array
-        Array of bead positions.
-
     """
     if seed != None:
         np.random.seed(seed)
@@ -79,7 +52,6 @@ def downsampled_trace(gamma = 1.e-5,
     -------
     xtrace_ds : numpy.array
         Array of bead positions.
-
     """
     bin_size = 1000 # points to average when downsampling
     fsim = int(bin_size*fsample) # simulation frequency
@@ -87,4 +59,5 @@ def downsampled_trace(gamma = 1.e-5,
     xtrace = simulate_trace(gamma,kappa,fsim,sim_points,seed) # do simulation
     bins = xtrace.reshape((N,bin_size)) # reshape into bins 
     xtrace_ds = np.mean(bins,axis=1) # average bins to produce downsampled trace
-    return xtrace_ds
\ No newline at end of file
+    return xtrace_ds
+
